{-----------------------------------------------------------------------------}
{   Code source  utilis dans la leon 6 du Guide Pratique de
{   la programmation de directshow sous delphi publi par Paul Glagla
{   sur le site http://paul.glagla.free.fr
{
{-----------------------------------------------------------------------------}
unit capture;

{-----------------------------------------------------------------------------}
interface

{-----------------------------------------------------------------------------}
uses
  SysUtils, Classes, DSPack,DSUtil,directshow9,windows;

{-----------------------------------------------------------------------------}
type
  TCaptureModule = class(TDataModule)
    CaptureGraph: TFilterGraph;
    CaptureFiltre: TFilter;
    DVSplitter: TFilter;
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  function GetCaptureSources(SourcesList:TStrings):Boolean;
  function SetCaptureSource(SourceName:string):Boolean;
  function buildpreviewgraph(vw:IBaseFilter):Boolean;
  function buildDV1graph (vw:IBaseFilter;fichier:widestring):Boolean;
  function buildDV2graph(vw:IBaseFilter;fichier:widestring):Boolean;
  function trouverFiltre(nom:string;var bf:IBaseFilter):Boolean;
end;

{-----------------------------------------------------------------------------}
var
  CaptureModule: TCaptureModule;

{-----------------------------------------------------------------------------}
implementation
{$R *.dfm}

{-----------------------------------------------------------------------------}
function TCaptureModule.GetCaptureSources(SourcesList:TStrings):Boolean;
{Notre fonction doit remplir une variable SourcesList de type TStrings
avec tous les noms de sources de capture trouves. Typiquement quand
vous appelez cette fonction, vous lui fournissez comme paramtre les items
d'une combobox de votre programme principal. De cette manire l'utilisateur
pourra choisir sa source en la slectionnant dans la combobox.}
var
i: integer;
SysDev:TSysDevEnum;
{TSysDevEnum est une classe dfinie dans DSutil, qui permet d'numrer
une catgorie de filtres prsents sur le systme. Voyez dans l'aide de DSPack
les proprits et mthodes de TSysDevEnum.}
begin
result:=false;
try
try
SourcesList.Clear;
SysDev:= TSysDevEnum.Create(CLSID_VideoInputDeviceCategory);
{Dans la mthode create de TSysDevEnum, on dfinit la catgorie des sources
de capture  numrer, ici par le paramtre CLSID_VideoInputDeviceCategory.
Vous pouvez essayer avec d'autres catgories, comme par exemple
CLSID_VideoCompressorCategory ou CLSID_AudioInputDeviceCategory etc...
qui sont dfinies dans l'unit directshow9.}
if SysDev.CountFilters > 0 then
for i := 0 to SysDev.CountFilters - 1 do
SourcesList.add(SysDev.Filters[i].FriendlyName);
result:=true;
except
end;
finally
SysDev.Free;
end;
end;

{-----------------------------------------------------------------------------}
function TCaptureModule.SetCaptureSource(SourceName:string):Boolean;
{Cette fonction prend comme paramtre un nom de source de capture.
C'est--dire typiquement l'un des noms chargs par la fonction
GetCaptureSources dans une liste.}
var
i:integer;
SysDev:TSysDevEnum;
begin
result:=False;
try
try
CaptureGraph.active:=false;
SysDev:= TSysDevEnum.Create(CLSID_VideoInputDeviceCategory);
for i:=0 to SysDev.CountFilters-1 do begin
if SysDev.Filters[i].FriendlyName=SourceName then break;
end;//for i
if i=SysDev.CountFilters then exit;
{Elle recharge un reprsentant de la classe TSysDevEnum avec tous les noms
des sources de capture prsentes sur le systme, et cherche dans la liste
celui qui porte le nom souhait.}
CaptureFiltre.BaseFilter.Moniker := SysDev.GetMoniker(i);
{Elle fixe alors la proprit BaseFilter de notre CaptureFiltre, en ayant
recours  la notion de Moniker, concept de la programmation COM qu'il serait
trop long d'expliquer ici. Si cela vous intresse, voir par exemple la thorie
fournie par Microsoft ici.}
CaptureGraph.Active:=true;
result:=true;
except
end;//except
finally
SysDev.Free;
end; //finally
end;

{-----------------------------------------------------------------------------}
function TCaptureModule.buildpreviewgraph(vw:IBaseFilter):Boolean;
{Cette fonction prend comme argument un IBaseFilter, qui est typiquement
la videowindow dans laquelle la prvisualisation doit intervenir.
Bien entendu, la proprit filtergraph de cette videowindow doit avoir t
fixe  CaptureGraph.On appelera la fonction par une instruction du type:
CaptureModule1.buildpreviewgraph(videowindow1 as IBaseFilter);
Le rsultat de la fonction est True si tout s'est bien pass, et False sinon.}
var hr:HResult;
{HResult est le type des codes d'erreur que fournit directshow.
En gnral quand une fonction directshow a bien march, elle renvoie
le message d'erreur S_Ok. Ici on verra cependant que le rsultat attendu
est diffrent.}
  fl:TFilterList;
  i:integer;
  fo:TFilterInfo;
begin
result:=false;
try
hr:=(CaptureGraph as ICaptureGraphBuilder2).RenderStream
  (@PIN_CATEGORY_PREVIEW,@MEDIATYPE_Interleaved,CaptureFiltre as IBaseFilter,
  nil,vw);
{Les paramtres de renderstream permettent d'abord d'indiquer  quelle
catgorie de pins on veut s'adresser: ici on prcise qu'on veut rendre un
pin de catgorie preview. On verra dans la leon 6 une autre catgorie,
qui est celle de capture. Le second paramtre prcise le type de Media
que l'on veut rendre. On trouverait @MEDIATYPE_Video ou @MEDIATYPE_Audio
sur une carte analogique. Mais ici on s'intresse  la broche DV A/V Out
qui donne vido et audio entrelaces. On choisit donc @MEDIATYPE_Interleaved.
Les trois paramtres suivants prcisent le filtre de dpart, ici bien
videmment c'est notre CaptureFiltre; puis un filtre intermdiaire
 utiliser, ici nil pour n'en imposer aucun; et enfin le filtre d'arrive,
ici vw qui est le IBaseFilter fourni par notre fonction buildpreviewgraph.}
if (hr<>VFW_S_NoPreviewpin) then exit;
{Aprs avoir appel renderstream, le code d'erreur attendu est
VFW_S_NoPreviewpin. En effet, renderstream part de notre CaptureFiltre
et y cherche une broche de type preview. Ce type de broches existe sur
les cartes analogiques, mais pas sur les sources DV. D'o un message
d'erreur, qui ne traduit en fait pas une erreur. Bien mieux, n'en trouvant
pas sur notre CaptureFiltre, renderstream se dbrouille pour en crer un en
ajoutant au graphe un filtre spcial qui s'appelle Smart Tee, et qui a pour
seule fonction de dupliquer le flux initial en un flux de type preview et
un flux de type capture. }
fl:=TFilterList.create(CaptureGraph as IFiltergraph);
{TFilterList est une classe qui permet d'numrer les filtres contenus
dans un graphe. Il fonctionne un peu comme TPinlist vu dans la leon
prcdente.}
for i:=0 to fl.count-1 do begin
fo:=fl.filterinfo[i];
{Filterinfo est une fonction qui s'applique  une TFilterList
(voir l'aide de DSPack) et qui met dans fo diffrentes informations sur le
filtre d'index i, en particulier une proprit achName qui n'est autre que
le nom familier du filtre.}
if fo.achName='DV Splitter' then break;
end;//for
if i<fl.Count then hr:=(CaptureGraph as ICaptureGraphBuilder2).
  RenderStream(nil,@MEDIATYPE_Audio,fl[i],nil,nil);
fl.Free;
{ Une fois qu'on a trouv le bon filtre, c''est--dire celui dont le nom
est DV Splitter, on appelle renderstream, en laissant la catgorie du pin 
nil, en prcisant le format du media comme tant de l'audio, et en prenant
bien sr notre filtre DV Splitter comme point de dpart, sans fixer de point
d'arrive qui est laiss  l'apprciation de directshow...}
CaptureGraph.Play;
result:=true;
except
end;//try except
end;

{-----------------------------------------------------------------------------}
procedure TCaptureModule.DataModuleDestroy(Sender: TObject);
begin
with capturegraph do begin
try
cleargraph;
active:=false
except
end;//try except
end;//with
end;

{-----------------------------------------------------------------------------}
function TCaptureModule.buildDV1graph (vw:IBaseFilter;fichier:widestring):Boolean;
 
{La fonction prend un paramtre qui est de nouveau le filtre de prvisualisation, comme vu dans la leon 5, puis le nom 
du fichier  enregistrer, avec son chemin. L'appel de la fonction se fait donc selon l'exemple suivant:
if capturemodule1.buildcapturegraph(videowindow1 as IbaseFilter,'f:\capture\trucmuche.avi') then begin...}
    var
        cg:ICaptureGraphBuilder2;
        avimux:IBaseFilter;
        writersink:IFileSinkFilter;
        hr:HResult;
{Les variables avimux et writersink sont des pointeurs sur des interfaces qui seront fournies par certaines fonctions de 
l'interface ICaptureGraphBuilder2.}
    begin
        result:=False;
    try
    try
        if not buildpreviewgraph(vw) then exit;
{On commence par appeler la fonction de preview vue  la leon 5. Si elle ne marche pas on quitte la fonction}
        capturegraph.stop;
        cg:=CaptureGraph as ICaptureGraphBuilder2;
        hr:=cg.SetOutputFileName(MEDIASUBTYPE_Avi,pwidechar(fichier),avimux, 
        writersink);
{La fonction SetOutputFileName de l'interface ICaptureGraphBuilder2 permet de fixer le nom du fichier de sortie. 
Nous lui donnons donc le widestring reu en paramtre. Elle fournit un IBaseFilter avimux qu'il faut garder pour l'utiliser 
plus loin dans renderstream. Voyez dans msdn la rfrence  de setoutputfilename.}
        if hr<>S_Ok then exit;
        hr:=cg.RenderStream(@PIN_CATEGORY_CAPTURE,@MEDIATYPE_Interleaved, 
        CaptureFiltre as IBaseFilter,nil,avimux);
{On appelle donc renderstream en demandant une broche de type Capture, et donnantun flux qui entrelace video et 
audio. On demande de construire le graphe en partant de la source CaptureFiltre, pour arriver au filtre avimux qui 
gnre l'criture sur disque (ce dernier continuera ensuite par le writersink cr par setoutputfilename pour dterminer 
le nom du fichier, mais il est inutile de le reprciser dans renderstream, car la connexion existe dj grce  
setoutputfilename}
        result:=succeeded(hr);
    except
    end;
    finally
        cg:=nil;
    end;
    end;

{-----------------------------------------------------------------------------}
function TCaptureModule.buildDV2graph(vw:IBaseFilter;fichier:widestring):Boolean;
{La fonction prend les mmes paramtre que buildDV1graph.}
    var 
        cg:ICaptureGraphBuilder2;
        avimux,smarttee:IBaseFilter;
        writersink:IFileSinkFilter;
        hr:HResult;
    begin
        result:=False;
    try
    try
        DVSplitter.filtergraph:=nil;
{Utile pour que notre filtre ne soit pas accapar par renderstream dans la partie preview}
        if not buildpreviewgraph(vw) then exit;
        capturegraph.stop;
        cg:=CaptureGraph as ICaptureGraphBuilder2;
        hr:=cg.SetOutputFileName(MEDIASUBTYPE_Avi, pwidechar(fichier), 
        avimux,writersink);
        if hr<>S_Ok then exit;
        if not trouverFiltre('Smart Tee',smarttee) then exit;
        DVSplitter.filtergraph:=capturegraph;
        hr:=cg.RenderStream(nil,nil,smarttee ,DVSplitter as IbaseFilter,avimux);
        if failed(hr) then exit;
        hr:=cg.renderstream(nil,@MEDIATYPE_Audio,DVSplitter,nil,avimux);
        if failed(hr) then exit;
        result:=true;
    except
    end;//try except
    finally
        cg:=nil;
    end;//try finally
    end;

{-----------------------------------------------------------------------------}
function TCaptureModule.trouverFiltre(nom:string;var bf:IBaseFilter):Boolean;
{Attention au var devant bf qui est indispensable pour qu'au retour bf pointe bien sur une interface IbaseFilter, malgr 
notre fl.free}
        var fl:TFilterList;
        i:integer;
    begin
        result:=False;
    try
    try
        fl:=TFilterlist.Create(capturegraph as IFilterGraph);
        if fl.Count=0 then exit;
        for i:=0 to fl.count-1 do begin
            if fl.filterinfo[i].achName=nom then break;
        end;//for
        if i<fl.Count then begin
            result:=true;
            bf:=fl[i];
        end;
    except
    end;//try except
    finally
        fl.Free;
    end;//try finally
    end;

{-----------------------------------------------------------------------------}
end.
